@@ -1,6 +1,6 @@
 /* dmcacheentrynwc.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2012-2017 Ralf Hoffmann.
+ * Copyright (C) 2012-2019 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -675,6 +675,39 @@ int DMCacheEntryNWC::removeEntry( int pos )
     return 0;
 }
 
+int DMCacheEntryNWC::updateActiveForRemoval( int pos )
+{
+    if ( pos < 0 || pos >= (int)m_entries.size() ) return 3;
+
+    if ( m_active_entry.dir_entry_id == m_entries.at( pos ).getEntryID() ) {
+        bool found = false;
+        for ( ; pos + 1 < (int)m_entries.size(); pos++ ) {
+            if ( m_entries.at( pos + 1 ).getUse() == true ) {
+                setActiveEntryPos( pos + 1 );
+                found = true;
+                break;
+            }
+        }
+
+        if ( ! found ) {
+            while ( true ) {
+                if ( pos > 0 &&
+                     m_entries.at( pos - 1 ).getUse() == true ) {
+                    setActiveEntryPos( pos - 1 );
+                    break;
+                } else if ( pos == 0 ) {
+                    setActiveEntryPos( -1 );
+                    break;
+                }
+
+                pos--;
+            }
+        }
+    }
+
+    return 0;
+}
+
 int DMCacheEntryNWC::temporarilyHideEntry( int pos )
 {
     if ( pos < 0 || pos >= (int)m_entries.size() ) return 1;
