@@ -696,7 +696,7 @@ void OwnOp::externprog( WPUContext *wpu, ActionMessage *msg )
             for ( int loopnr = 1;; loopnr++ ) {
                 std::string res_str;
                 if ( wpu->parse( com_str, res_str, EXE_STRING_LEN - strlen( tmpoutput ) - 3,
-                                 true, WPUContext::PERSIST_ALL ) == WPUContext::PARSE_SUCCESS ) {
+                                 true, WPUContext::PERSIST_FLAGS ) == WPUContext::PARSE_SUCCESS ) {
                     string1 = Worker_secureCommandForShell( res_str.c_str() );
                     if ( string1.length() > 0 ) {
                         if ( ownstart == OWNOP_SHOW_OUTPUT ||
@@ -711,6 +711,11 @@ void OwnOp::externprog( WPUContext *wpu, ActionMessage *msg )
                         }
                         fp->putString( "\n" );
                     }
+
+                    msg->getWorker()->storePathPers( wpu->getPathDBFilename(),
+                                                     res_str,
+                                                     time( NULL ) );
+                    
                     if ( separate_each_entry == false ) break;
                     if ( wpu->filelistRealSize( false ) == 0 ) break;
                     if ( wpu->filelistRealSize( false ) == lastsize[0] ) {
