@@ -1,6 +1,6 @@
 /* startprogop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2014 Ralf Hoffmann.
+ * Copyright (C) 2001-2019 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -48,6 +48,7 @@
 #include "pers_string_list.hh"
 #include "wpucontext.h"
 #include "ownop.h"
+#include "ajson.hh"
 
 const char *StartProgOp::name="StartProgOp";
 
@@ -214,6 +215,12 @@ StartProgOp::startprog( WPUContext *wpu, ActionMessage *am )
               default_file = name_string;
           }
       }
+
+      if ( tfe->filetype ) {
+          m_default_file_filetype = tfe->filetype->getName();
+      } else {
+          m_default_file_filetype = "";
+      }
   } else {
     default_file = "";
   }
@@ -268,6 +275,20 @@ StartProgOp::startprog( WPUContext *wpu, ActionMessage *am )
                          true,
                          WPUContext::PERSIST_FLAGS ) == WPUContext::PARSE_SUCCESS ) {
             string1 = Worker_secureCommandForShell( res_str.c_str() );
+
+            if ( m1->_global ) {
+                // as for now only store global programs and the file
+                // in the persistent database
+                if ( wpu->getPathDBFilename().empty() ) {
+                    am->getWorker()->storePathPers( fullname,
+                                                    res_str,
+                                                    time( NULL ) );
+                } else {
+                    am->getWorker()->storePathPers( wpu->getPathDBFilename(),
+                                                    res_str,
+                                                    time( NULL ) );
+                }
+            }
         }
 
         if ( string1.length() > 0 ) {
@@ -778,6 +799,8 @@ std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
                                          cmin );
   m_completion_lv->setHBarState( 2 );
   m_completion_lv->setVBarState( 2 );
+  m_completion_lv->setNrOfFields( 2 );
+  m_completion_lv->setGlobalFieldSpace( 5 );
 
   AWindow *subwin11_2 = new AWindow( aguix, 0, 0, 10, 10, "" );
   acsw11->add( subwin11_2, 0, 2, cmin );
@@ -858,7 +881,14 @@ std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
   m_completion_enabled = true;
   m_previous_sg_content = "";
 
-  updateSGFromComplete( m_cmd_sg, history, m_completion_lv );
+  cfgfile += "-filetype.json";
+
+  auto filetype_history = AJSON::load( cfgfile );
+  if ( ! AJSON::as_object( filetype_history ) ) {
+      filetype_history = AJSON::make_object();
+  }
+
+  updateSGFromComplete( m_cmd_sg, history, filetype_history, m_completion_lv );
 
   AGMessage *msg;
   while((msg=aguix->GetMessage(NULL))!=NULL) aguix->ReplyMessage(msg);
@@ -894,7 +924,7 @@ std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
 	      arcw->_sg->setText( arcw->_lv->getText( row, 0 ).c_str() );
             }
           } else if ( msg->fieldlv.lv == m_completion_lv ) {
-              updateSGFromRow( msg->fieldlv.row, history );
+              updateSGFromRow( msg->fieldlv.row, history, filetype_history );
           }
           break;
 	case AG_STRINGGADGET_OK:
@@ -911,7 +941,7 @@ std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
         if ( msg->stringgadget.sg == ftw->m_filter_sg ) {
             ftw->updateView();
         } else if ( msg->stringgadget.sg == m_cmd_sg ) {
-            updateSGFromComplete( m_cmd_sg, history, m_completion_lv );
+            updateSGFromComplete( m_cmd_sg, history, filetype_history, m_completion_lv );
         }
         break;
         case AG_KEYPRESSED:
@@ -962,7 +992,7 @@ std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
                         m_completion_lv->setActiveRow( trow );
                         m_completion_lv->showActive();
 
-                        updateSGFromRow( trow, history );
+                        updateSGFromRow( trow, history, filetype_history );
                     }
                     break;
                 case XK_Up:
@@ -976,7 +1006,7 @@ std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
                         m_completion_lv->setActiveRow( trow );
                         m_completion_lv->showActive();
 
-                        updateSGFromRow( trow, history );
+                        updateSGFromRow( trow, history, filetype_history );
                     }
                     break;
             }
@@ -1004,6 +1034,8 @@ std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
                   if ( base[0] != ' ' ) {
                       history.removeEntry( base );
                       history.pushFrontEntry( base );
+
+                      updateFiletypeHistory( filetype_history, base );
                   }
               }
           } else {
@@ -1015,6 +1047,8 @@ std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
                       if ( base[0] != ' ' ) {
                           history.removeEntry( base );
                           history.pushFrontEntry( base );
+
+                          updateFiletypeHistory( filetype_history, base );
                       }
                   }
               }
@@ -1041,6 +1075,8 @@ std::unique_ptr< StartProgOp::StartSettings > StartProgOp::showGUI()
           sets->_global = m1->_gcb->getState();
           sets->_inbackground = m1->_ibcb->getState();
           sets->_dontcd = m1->_dcdcb->getState();
+
+          AJSON::dump( filetype_history, cfgfile );
       } else if ( k1->getCurOption() == 1 ) {
           sets->_startmode = StartSettings::HANDLE_TYPE;
 
@@ -1200,24 +1236,57 @@ void StartProgOp::setGUIMsg( std::string msg )
 }
 
 void StartProgOp::updateCompletionList( const std::string &base,
-                                        PersistentStringList &history )
+                                        PersistentStringList &history,
+                                        std::shared_ptr< AJSON::JSONType > &filetype_history )
 {
 
    std::list< std::string > l;
+   std::set< std::string > commands_in_use;
 
    l = history.getList();
 
    m_current_completions.clear();
 
+   auto jobj = AJSON::as_object( filetype_history );
+
+   if ( jobj ) {
+       auto jft = AJSON::as_object( jobj->get( "filetype" ) );
+
+       if ( jft ) {
+           auto jcommands = AJSON::as_array( jft->get( m_default_file_filetype ) );
+
+           if ( jcommands ) {
+               for ( size_t pos = 0; pos < jcommands->size(); pos++ ) {
+                   auto jcommand = AJSON::as_object( jcommands->get( pos ) );
+
+                   if ( ! jcommand ) continue;
+
+                   auto jcmd = AJSON::as_string( jcommand->get( "command" ) );
+
+                   if ( ! jcmd ) continue;
+
+                   if ( AGUIXUtils::starts_with( jcmd->get_value(), base ) ) {
+                       m_current_completions.push_back( std::make_pair( FILETYPE_COMPLETION,
+                                                                        jcmd->get_value() ) );
+                       commands_in_use.insert( jcmd->get_value() );
+                   }
+               }
+           }
+       }
+   }
+   
    for ( auto &s : l ) {
-       if ( AGUIXUtils::starts_with( s, base ) ) {
-           m_current_completions.push_back( s );
+       if ( commands_in_use.count( s ) == 0 ) {
+           if ( AGUIXUtils::starts_with( s, base ) ) {
+               m_current_completions.push_back( std::make_pair( ANY_COMPLETION, s ) );
+           }
        }
    }
 }
 
 void StartProgOp::updateSGFromComplete( StringGadget *sg,
                                         PersistentStringList &history,
+                                        std::shared_ptr< AJSON::JSONType > &filetype_history,
                                         FieldListView *lv )
 {
     std::string old_base;
@@ -1230,19 +1299,25 @@ void StartProgOp::updateSGFromComplete( StringGadget *sg,
         prefix.resize( sg->getCursor() );
     }
     
-    updateCompletionList( prefix, history );
+    updateCompletionList( prefix, history, filetype_history );
 
     lv->setSize( m_current_completions.size() );
 
     int row = 0;
     for ( auto &s : m_current_completions ) {
-        std::string s2( s );
+        std::string s2( s.second );
 
         if ( ! AGUIXUtils::ends_with( s2, "}" ) ) {
             s2 += " ";
             s2 += m_current_original_file;
         }
 
+        if ( s.first == FILETYPE_COMPLETION ) {
+            lv->setText( row, 1, catalog.getLocale( 1321 ) );
+        } else {
+            lv->setText( row, 1, catalog.getLocale( 1322 ) );
+        }
+
         lv->setText( row, 0, s2 );
         lv->setPreColors( row, FieldListView::PRECOLOR_ONLYACTIVE );
 
@@ -1276,7 +1351,7 @@ void StartProgOp::updateSGFromComplete( StringGadget *sg,
 
     if ( sg->getCursor() > 0 ) {
 
-        std::string new_base = m_current_completions.front();
+        std::string new_base = m_current_completions.front().second;
 
         std::string new_text( new_base );
 
@@ -1301,7 +1376,8 @@ void StartProgOp::updateSGFromComplete( StringGadget *sg,
 }
 
 void StartProgOp::updateSGFromRow( int row,
-                                   PersistentStringList &history )
+                                   PersistentStringList &history,
+                                   std::shared_ptr< AJSON::JSONType > &filetype_history )
 {
     std::string t = m_completion_lv->getText( row, 0 );
 
@@ -1312,6 +1388,47 @@ void StartProgOp::updateSGFromRow( int row,
 
     m_completion_enabled = false;
 
-    updateSGFromComplete( m_cmd_sg, history, m_completion_lv );
+    updateSGFromComplete( m_cmd_sg, history, filetype_history, m_completion_lv );
 }
 
+void StartProgOp::updateFiletypeHistory( std::shared_ptr< AJSON::JSONType > &filetype_history,
+                                         const std::string &base )
+{
+    auto jobj = AJSON::as_object( filetype_history );
+
+    if ( ! jobj ) return;
+
+    auto jft = AJSON::as_object( jobj->get( "filetype" ) );
+
+    if ( ! jft ) {
+        jft = AJSON::as_object( jobj->set( "filetype", AJSON::make_object() ) );
+
+        if ( ! jft ) return;
+    }
+
+    auto jold_commands = AJSON::as_array( jft->get( m_default_file_filetype ) );
+
+    auto jnew_commands = AJSON::make_array();
+
+    auto jnew_command = AJSON::make_object();
+    jnew_command->set( "command", AJSON::make_string( base ) );
+    jnew_commands->append( jnew_command );
+
+    if ( jold_commands ) {
+        for ( size_t pos = 0; pos < jold_commands->size(); pos++ ) {
+            auto jold_command = AJSON::as_object( jold_commands->get( pos ) );
+
+            if ( ! jold_command ) continue;
+
+            auto jcmd = AJSON::as_string( jold_command->get( "command" ) );
+
+            if ( ! jcmd ) continue;
+
+            if ( jcmd->get_value() == base ) continue;
+
+            jnew_commands->append( jold_command );
+        }
+    }
+
+    jft->set( m_default_file_filetype, jnew_commands );
+}
