@@ -210,14 +210,21 @@ public:
     DeepPathStore &getPathStore();
 
     bool storePathPers( const std::string &path, time_t ts );
+    bool storePathPers( const std::string &path,
+                        const std::string &command_str,
+                        time_t ts );
     void removePathPers( const std::string &path );
+    void removePathProgramPers( const std::string &program );
     DeepPathStore getPathsPers();
+    std::map< std::string, DeepPathStore > getPathProgramsPers();
+    loff_t getSizeOfPersPaths() const;
+    loff_t getSizeOfPersPathPrograms() const;
     void relocateEntriesPers( const std::string &dest,
                               const std::string &source,
                               time_t ts,
                               bool move );
 
-    void storePathPersIfInProgress( const std::string &path );
+    bool storePathPersIfInProgress( const std::string &path );
     void setPathJumpInProgress( bool nv = true );
     void setPathJumpInProgress( const std::string last_filter );
 
