@@ -27,7 +27,7 @@
 #include "nwc_path.hh"
 #include "datei.h"
 
-#define CATENTRY 1321
+#define CATENTRY 1328
 #define CATFLAGENTRY 59
 
 WorkerLocale catalog;
@@ -1422,6 +1422,13 @@ The default value here is the active entry and it's already correctly protected!
   catalog[1318]=dupstring( "Select command" );
   catalog[1319]=dupstring( "Description" );
   catalog[1320]=dupstring( "Remove non-existing entries from the database. A non-zero 'unused days' value will only remove non-existing entries not used for longer than the given number of days." );
+  catalog[1321]=dupstring( "(Filetype)" );
+  catalog[1322]=dupstring( "(All)" );
+  catalog[1323]=dupstring( "<enter filter>" );
+  catalog[1324]=dupstring( "Clean unused programs" );
+  catalog[1325]=dupstring( "Program" );
+  catalog[1326]=dupstring( "Last accesses by external program - F3" );
+  catalog[1327]=dupstring( "Show accesses by external program" );
 
 //  catalog[]=dupstring( "" );
   
